<?php
// $Id: XmlLib.inc.php,v 1.1 2010/11/15 14:50:12 hylkevds Exp $

/**
 * @file
 * Collection of XML methods for handling text nodes.
 */

/**
 * Copy all the attributes of $node into a string as it would be used in html,
 * and return that string.
 *
 * @param DOMNode $node
 *   XML node to copy all the attributes from.
 *
 * @return string containing the attributes.
 */
function cq_copy_attributes($node) {
  $retval .= '';
  $attribs = $node->attributes;
  foreach ($node->attributes as $attr_name => $attr_node) {
    $retval .= ' ' . $attr_name . '="' . $attr_node->nodeValue . '"';
  }
  return $retval;
}

/**
 * Recursively parse the children of an XML node and return the parts that are
 * valid html.  The context CqQuestionInterface object can provide a list of node names
 * that are not html, but can be turned into html by the context object.
 *
 * If fullFilter is set, the final html will be filtered through the Drupal
 * filter set of the context CqQuestionInterface.
 *
 * @param DOMNode $parent
 *   The XML node to parse the children of.
 * @param CqQuestionInterface $context
 *   The question that can supply extra node names to handle, and drupal
 *   filters.
 * @param boolean $fullFilter
 *   Filter the result also through the drupal content filters?
 *
 * @return string
 *   The html contained in the node.
 */
function cq_get_text_content($parent, $context, $full_filter = TRUE) {
  $retval = '';
  $nodes = $parent->childNodes;
  foreach ($nodes as $node) {
    switch ($node->nodeName) {
      case '#comment':
        break;

      case '#text':
      case '#cdata-section':
        $retval .= $node->nodeValue;
        break;

      case 'formblock':
        $retval .= '<formblock/>';
        break;

      case 'img':
      case 'br':
        $retval .= '<' . $node->nodeName . ' ' . cq_copy_attributes($node) . '/>';
        break;

      case 'sub':
      case 'sup':
      case 'p':
      case 'pre':
      case 'b':
      case 'i':
      case 'u':
      case 'strong':
      case 'em':
        $retval .= '<' . $node->nodeName . '>' . cq_get_text_content($node, $context, FALSE) . '</' . $node->nodeName . '>';
        break;

      case 'table':
      case 'a':
      case 'tr':
      case 'td':
      case 'th':
      case 'span':
      case 'div':
      case 'ul':
      case 'ol':
      case 'dl':
      case 'dt':
      case 'dd':
      case 'li':
        $retval .= '<' . $node->nodeName . ' ' . cq_copy_attributes($node) . '>' . cq_get_text_content($node, $context, FALSE) . '</' . $node->nodeName . '>';
        break;

      default:
        $retval .= $node->nodeValue;
        break;
    }
    foreach ($context->getHandledTags() AS $tag) {
      if (drupal_strtolower($node->nodeName) == drupal_strtolower($tag)) {
        $retval .= $context->handleTag($node);
      }
    }
  }
  if ($full_filter) {
    $drupal_node = $context->getNode();
    $retval = check_markup(trim($retval), $drupal_node->format, FALSE);
  }
  return $retval;
}
